'Clipboard Viewer, a utility to examine raw clipboard contents.
'Copyright (C) 2008 Nathan Tuggy <mailto:nathan@tuggycomputer.com>
'
'This program is free software: you can redistribute it and/or modify
'it under the terms of the GNU General Public License as published by
'the Free Software Foundation, either version 3 of the License, or
'(at your option) any later version.
'
'This program is distributed in the hope that it will be useful,
'but WITHOUT ANY WARRANTY; without even the implied warranty of
'MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
'GNU General Public License for more details.
'
'You should have received a copy of the GNU General Public License
'along with this program.  If not, see <http://www.gnu.org/licenses/>.

Imports System.Windows.Forms
Imports System.Text.RegularExpressions

Module Main

    Sub Main()
        Const prxfcSwitchFlags As RegexOptions = RegexOptions.IgnoreCase Or RegexOptions.ExplicitCapture
        Dim prxpAllFormats As New Regex(" ?/f(ormats)?", prxfcSwitchFlags)
        Dim prxpAllData As New Regex(" ?/all(data)?", prxfcSwitchFlags)
        Dim prxpForceDataDisplay As New Regex(" ?/force", prxfcSwitchFlags)
        Dim prxpHelp As New Regex(" ?/(\?|h|help)", prxfcSwitchFlags)
        Dim prxpQuiet As New Regex(" ?/q(uiet)?", prxfcSwitchFlags)
        Dim prxpWait As New Regex(" ?/w(ait)?", prxfcSwitchFlags)
        Dim pstrCommand As String = Command()

        ' Check consistency
        If (prxpQuiet.IsMatch(pstrCommand) AndAlso prxpWait.IsMatch(pstrCommand)) Then
            Console.WriteLine("Do not specify both /quiet and /wait. (Specify /? to see available options.)")
        ElseIf (prxpAllFormats.IsMatch(pstrCommand) AndAlso prxpAllData.IsMatch(pstrCommand)) Then
            Console.WriteLine("Do not specify both /formats and /alldata. (Specify /? to see available options.)")
        Else
            ' Display data
            Dim pblnForceDataDisplay As Boolean = prxpForceDataDisplay.IsMatch(pstrCommand)
            If pblnForceDataDisplay Then pstrCommand = prxpForceDataDisplay.Replace(pstrCommand, "")

            If prxpHelp.IsMatch(pstrCommand) Then
                ShowHelp()
            ElseIf pstrCommand.Length = 0 OrElse prxpAllFormats.IsMatch(pstrCommand) Then
                WriteAvailableFormats()
            ElseIf prxpAllData.IsMatch(pstrCommand) Then
                WriteAvailableFormatData(pblnForceDataDisplay)
            Else
                WriteFormatData(pstrCommand, pblnForceDataDisplay)
            End If

            ' Wait for user, or close immediately if desired
            If prxpQuiet.IsMatch(pstrCommand) Then
                Exit Sub
            Else
                Console.Write("Press any key to continue...")
                Console.ReadKey()
            End If
        End If
    End Sub

    Sub ShowHelp()
        Console.Write(My.Resources.Help)
    End Sub

    Sub WriteFormatData(ByVal Format As String, ByVal ForceDataDisplay As Boolean)
        Const pstrcDescriptionPrefix As String = vbCrLf + vbTab
        Const pstrcDescription As String = pstrcDescriptionPrefix + "Format: '{0}'"

        Dim pdto As IDataObject = Clipboard.GetDataObject()
        Dim pstrDescription As String = String.Format(pstrcDescription, Format)
        Console.WriteLine(pstrDescription)
        Dim pstrSeparator As String = New String("-"c, pstrDescription.Length + 2 - pstrcDescriptionPrefix.Length)
        Console.WriteLine(vbTab + pstrSeparator)

        If Not pdto.GetDataPresent(Format) Then
            Console.WriteLine("<no data available>")
        Else
            Dim pobjData As Object = pdto.GetData(Format)
            WriteData(pobjData, ForceDataDisplay)
        End If
    End Sub

    Sub WriteData(ByVal obj As Object, ByVal ForceDataDisplay As Boolean)
        If TypeOf obj Is String Then
            Console.WriteLine(obj)
        ElseIf TypeOf obj Is IEnumerable Then
            For Each pobj As Object In DirectCast(obj, IEnumerable)
                WriteData(pobj, ForceDataDisplay)
            Next
        ElseIf obj IsNot Nothing And Not ForceDataDisplay Then
            Console.WriteLine("<{0}>", obj.GetType().FullName)
        ElseIf obj IsNot Nothing Then
            Console.WriteLine(GetFormatData(obj))
        Else
            Console.WriteLine()
        End If
    End Sub

    Sub WriteAvailableFormatData(ByVal ForceDataDisplay As Boolean)
        Dim pdto As IDataObject = Clipboard.GetDataObject()
        For Each pstrFormat As String In pdto.GetFormats()
            WriteFormatData(pstrFormat, ForceDataDisplay)
        Next pstrFormat
    End Sub

    Sub WriteAvailableFormats()
        Dim pdto As IDataObject = Clipboard.GetDataObject()
        Dim pstrDescription As String = "Available Formats:"
        Console.WriteLine(vbTab + pstrDescription)
        Dim pstrSeparator As String = StrDup(pstrDescription.Length + 2, "-")
        Console.WriteLine(vbTab + pstrSeparator)
        For Each pstrFormat As String In pdto.GetFormats()
            Console.WriteLine(pstrFormat)
        Next pstrFormat
    End Sub

    Function GetFormatData(ByVal Data As IO.MemoryStream) As String
        Dim pabytcUnicodeBEBOM() As Byte = {&HFE, &HFF}
        Dim pabytcUnicodeLEBOM() As Byte = {&HFF, &HFE}
        Dim pabyt(2) As Byte
        Data.Read(pabyt, 0, 2)
        If pabyt(0) = pabytcUnicodeBEBOM(0) AndAlso pabyt(1) = pabytcUnicodeBEBOM(1) Then
            Return Text.Encoding.BigEndianUnicode.GetString(Data.ToArray())
        ElseIf pabyt(0) = pabytcUnicodeLEBOM(0) AndAlso pabyt(1) = pabytcUnicodeLEBOM(1) Then
            Return Text.Encoding.Unicode.GetString(Data.ToArray())
        Else
            Dim bencUTF8 As New Text.UTF8Encoding(False, True)
            Return Text.Encoding.UTF8.GetString(Data.ToArray())
        End If
    End Function

    Function GetFormatData(ByVal Data As Object) As String
        If TypeOf Data Is IO.MemoryStream Then Return GetFormatData(CType(Data, IO.MemoryStream))
        Return String.Format("<{0}>", Data.GetType().FullName)
    End Function
End Module
